
static const int		MAX_MATRICES = 16;				// s̍ő吔
float4x4				mBoneMatrices[MAX_MATRICES];	// {[̍s
bool					bEnableSkinning = false;		// XLjO
shared float4x4			mProj;

float4x4				mWorldView;		// [h x r[
float					fBaseAlpha;		// At@l̊{l
float					fFocus;			// _܂ł̋
float					fDOF;			// ͈
float					fWeights[8];	// d
float2					vOffsets[8];	// eNX`W̃ItZbg
sampler2D				sampler0;
sampler2D				samplerDepth;
sampler2D				samplerBlur;
sampler2D				samplerScene;


//=====================================================================
// VS: [x`悷
//=====================================================================
void VS_DrawDepth(float4		inPosition		: POSITION,
				  float2		inTex			: TEXCOORD,
				  float3		inWeights		: BLENDWEIGHT,
				  float4		inIndices		: BLENDINDICES,
				  out float4	outPosition		: POSITION,
				  out float		outDepth		: TEXCOORD0,
				  out float2	outTex			: TEXCOORD1)
{
	
	// XLjO̗LŃu`
	if (bEnableSkinning)
	{
		float4 worldPos = mul(inPosition, mBoneMatrices[inIndices.x]) * inWeights.x + 
							mul(inPosition, mBoneMatrices[inIndices.y]) * inWeights.y + 
							mul(inPosition, mBoneMatrices[inIndices.z]) * inWeights.z + 
							mul(inPosition, mBoneMatrices[inIndices.w]) * (1.0 - inWeights.x - inWeights.y - inWeights.z);
		outPosition = mul(worldPos, mWorldView);
	}
	else
	{
		outPosition = mul(inPosition, mWorldView);
	}

	outDepth = (outPosition.z - fFocus) * fDOF;
	outPosition = mul(outPosition, mProj);
	outTex = inTex;
}


//=====================================================================
// PS: [x`悷
//=====================================================================
float4 PS_DrawDepth(float depth: TEXCOORD0, float2 tex: TEXCOORD1) : COLOR0
{
	float3 d = 1.0 - abs(depth);
	return float4(d, tex2D(sampler0, tex).a + fBaseAlpha);
}


//=====================================================================
// VS: KEXڂ
//=====================================================================
void VS_Blur(float4			inPosition		: POSITION,
			 float2			inTex			: TEXCOORD,
			 out float4		outPosition		: POSITION,
			 out float2		outTex0			: TEXCOORD0)
{
	outPosition = inPosition;
	outTex0 = inTex;
}


//=====================================================================
// PS: KEXڂ
//=====================================================================
float4 PS_Blur(float2 tex: TEXCOORD0) : COLOR0
{
	float4 ret;

	ret =  tex2D(sampler0, tex) * fWeights[0];
	ret += (tex2D(sampler0, tex + vOffsets[1]) + tex2D(sampler0, tex - vOffsets[1])) * fWeights[1];
	ret += (tex2D(sampler0, tex + vOffsets[2]) + tex2D(sampler0, tex - vOffsets[2])) * fWeights[2];
	ret += (tex2D(sampler0, tex + vOffsets[3]) + tex2D(sampler0, tex - vOffsets[3])) * fWeights[3];
	ret += (tex2D(sampler0, tex + vOffsets[4]) + tex2D(sampler0, tex - vOffsets[4])) * fWeights[4];
	ret += (tex2D(sampler0, tex + vOffsets[5]) + tex2D(sampler0, tex - vOffsets[5])) * fWeights[5];
	ret += (tex2D(sampler0, tex + vOffsets[6]) + tex2D(sampler0, tex - vOffsets[6])) * fWeights[6];
	ret += (tex2D(sampler0, tex + vOffsets[7]) + tex2D(sampler0, tex - vOffsets[7])) * fWeights[7];

	return ret;
}


//=====================================================================
// VS: R|Wbg
//=====================================================================
void VS_Composite(float4			inPosition		: POSITION,
				  float2			inTex			: TEXCOORD,
				  out float4		outPosition		: POSITION,
				  out float2		outTex0			: TEXCOORD0)
{
	outPosition = inPosition;
	outTex0 = inTex;
}


//=====================================================================
// PS: R|Wbg
//=====================================================================
float4 PS_Composite(float2 tex	: TEXCOORD0) : COLOR0
{
	float t = tex2D(samplerDepth, tex);
	float4 a = tex2D(samplerScene, tex);
	float4 b = tex2D(samplerBlur, tex);
	return lerp(a, b, t);
}


//=====================================================================
// [x`悷eNjbN
//=====================================================================
technique DrawDepth
{
	pass
	{
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATEREQUAL;
		AlphaRef = 1;

		VertexShader = compile vs_2_0 VS_DrawDepth();
		PixelShader = compile ps_2_0 PS_DrawDepth();
	}
}


//=====================================================================
// eNX`ڂeNjbN
//=====================================================================
technique Blur
{
	pass
	{
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = FALSE;
		ZEnable = FALSE;
		MinFilter[0] = LINEAR;
		MagFilter[0] = LINEAR;
		AddressU[0] = CLAMP;
		AddressV[0] = CLAMP;

		VertexShader = compile vs_2_0 VS_Blur();
		PixelShader = compile ps_2_0 PS_Blur();
	}
}


//=====================================================================
// R|Wbg
//=====================================================================
technique Composite
{
	pass
	{
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = FALSE;
		ZEnable = FALSE;

		MagFilter[0] = LINEAR;
		MagFilter[1] = LINEAR;
		MagFilter[2] = LINEAR;

		AddressU[0] = CLAMP;
		AddressV[0] = CLAMP;
		AddressU[1] = CLAMP;
		AddressV[1] = CLAMP;
		AddressU[2] = CLAMP;
		AddressV[2] = CLAMP;

		Sampler[0] = samplerDepth;
		Sampler[1] = samplerScene;
		Sampler[2] = samplerBlur;

		VertexShader = compile vs_2_0 VS_Composite();
		PixelShader = compile ps_2_0 PS_Composite();
	}
}


