import Blender
import struct
from Blender import *
from Blender.Mathutils import *
from struct import *


def FileSelectorCallback(filename):
	object = Object.GetSelected()[0]
	
	# Ensure selected object is mesh
	if object.type != 'Mesh':
		print object, 'is not Mesh'
		return
	
	# Ensure mesh has armature
	if object.parentType != Object.ParentTypes.ARMATURE:
		print object, "must have armature"
		return
	
	# a matrix that converts YZ axes
	matTransform = RotationMatrix(-90, 4, 'x')
	
	# a matrix that flips Z axis
	matZFlip = ScaleMatrix(-1, 4, Vector([0, 0, 1]))
	
	armObject = object.parent
	armData = armObject.data
	pose = armObject.getPose()
	totalFrameCount = 0
	totalChannelCount = 0
	fps = Scene.GetCurrent().getRenderingContext().fps
	
	# Make list of valid actions to export
	actionList = []
	for action in Armature.NLA.GetActions().values():
		if len(action.getFrameNumbers()):
			actionList.append(action)
	
	# count # of frames
	for action in actionList:
		totalFrameCount += len(action.getAllChannelIpos()) * len(action.getFrameNumbers())
	
	# count # of channels
	for action in actionList:
		totalChannelCount += len(action.getAllChannelIpos())
	
	# open file
	file = open(filename, "wb")
	
	#
	# FILE HEADER
	#
	#   long  version
	#   long  frameCount
	#   long  channelCount
	#   long  animationCount
	#
	file.write(pack(
		'llll', 
		1, 
		totalFrameCount,
		totalChannelCount,
		len(actionList)
		))
	
	# export all key frames in array
	for action in actionList:
		
		action.setActive(armObject)

		# for each ipo channels in action
		for channelName in action.getChannelNames():
			
			bone = pose.bones[channelName]
			prevFrame = 1
			
			# for each key points in action
			for frame in action.getFrameNumbers():
				
				Blender.Set('curframe', frame)
				
				matrix = bone.poseMatrix.copy()
				
				# if bone has parent, calculate local matrix of bone.
				# otherwise use pose matrix in world space
				if bone.parent:
					matrix *= pose.bones[bone.parent.name].poseMatrix.copy().invert()
				else:
					matrix *= armObject.matrix * matTransform
				
				# convert matrix coordinate system to left handed
				matrix = matZFlip * matrix * matZFlip
			
				# scale XYZ
				S = matrix.scalePart()

				# quat XYZW
				R = matrix.toQuat()
			
				# translation XYZ
				T = matrix.translationPart()
				
				#
				# KEY FRAME
				#
				#   float    deltaTime
				#   float[3] scale
				#   float[4] rotation
				#   float[3] position
				#
				file.write(pack('fffffffffff', 
					float(frame - prevFrame) / fps, 
					S.x, S.z, S.y,
					R.x, R.y, R.z, R.w,
					T.x, T.y, T.z))
				
				prevFrame = frame


	# export all channels
	frameCount = 0
	for action in actionList:
		action.setActive(armObject)
		
		#
		# CHANNEL HEADER
		#
		#   char[32]  channelName
		#   long      frameStart
		#   long      frameCount
		#
		
		# for each ipo channels in action
		for channelName in action.getChannelNames():
	
			startFrame = frameCount
			endFrame = startFrame + len(action.getFrameNumbers())
			print action.name, endFrame
			file.write(pack('32sll', 
				channelName,
				startFrame,
				endFrame - startFrame))
			
			frameCount = endFrame


	# export all actions
	channelCount = 0
	for action in actionList:
		action.setActive(armObject)
		actionName = action.name
		actionName.upper()
			
		#
		# ANIMATION HEADER
		#
		#   char[32]  animationName
		#   long      channelStart
		#   long      channelCount
		#   float     length
		#
		startChannel = channelCount
		endChannel = startChannel + len(action.getChannelNames())
		animationLength = float(action.getFrameNumbers()[-1] - 1) / fps
		
		file.write(pack('32sllf', 
			actionName,
			startChannel,
			endChannel - startChannel,
			animationLength))
			
		print "Animation: %s (%.2fsecs)\n" % (actionName, animationLength)
			
		channelCount = endChannel

	
	# close file
	file.close()
	
	
	

Window.FileSelector(FileSelectorCallback, "OK", Blender.sys.makename(ext='.a'))