import Blender
from Blender import *
from Blender.Mathutils import *
from struct import *


D3DFVF_XYZ              = 0x002
D3DFVF_XYZRHW           = 0x004
D3DFVF_XYZB1            = 0x006
D3DFVF_XYZB2            = 0x008
D3DFVF_XYZB3            = 0x00a
D3DFVF_XYZB4            = 0x00c
D3DFVF_XYZB5            = 0x00e
D3DFVF_XYZW             = 0x4002
D3DFVF_NORMAL           = 0x010
D3DFVF_PSIZE            = 0x020
D3DFVF_DIFFUSE          = 0x040
D3DFVF_SPECULAR         = 0x080
D3DFVF_TEX0             = 0x000
D3DFVF_TEX1             = 0x100
D3DFVF_TEX2             = 0x200
D3DFVF_TEX3             = 0x300
D3DFVF_TEX4             = 0x400
D3DFVF_TEX5             = 0x500
D3DFVF_TEX6             = 0x600
D3DFVF_TEX7             = 0x700
D3DFVF_TEX8             = 0x800
D3DFVF_LASTBETA_UBYTE4  = 0x1000


class Direct3DVertex:
	def __init__(self, fvf):
		self.fvf = fvf
		self.position = Vector(0, 0, 0, 0)
		self.normal = Vector(0, 0, 0)
		self.diffuse = 0xFFFFFFFF
		self.specular = 0xFFFFFFFF
		self.weight = [0, 0, 0, 0, 0]
		self.texcoord = []
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
	def getBytes(self):
		ret = ""
		
		if self.fvf & D3DFVF_XYZ == D3DFVF_XYZ:
			ret += pack("fff", 
				self.position.x, 
				self.position.y, 
				self.position.z)
		
		if self.fvf & D3DFVF_XYZW == D3DFVF_XYZW:
			ret += pack("f", self.position.w)
		
		wCount = 0
		if self.fvf & D3DFVF_XYZB5 == D3DFVF_XYZB5: wCount = 5
		elif self.fvf & D3DFVF_XYZB4 == D3DFVF_XYZB4: wCount = 4
		elif self.fvf & D3DFVF_XYZB3 == D3DFVF_XYZB3: wCount = 3
		elif self.fvf & D3DFVF_XYZB2 == D3DFVF_XYZB2: wCount = 2
		elif self.fvf & D3DFVF_XYZB1 == D3DFVF_XYZB1: wCount = 1

		for i in range(wCount):
			ret += pack("f", self.weight[i])
		
		if self.fvf & D3DFVF_NORMAL == D3DFVF_NORMAL:
			ret += pack("fff", 
				self.normal.x, 
				self.normal.y, 
				self.normal.z)
		
		if self.fvf & D3DFVF_DIFFUSE == D3DFVF_DIFFUSE:
			ret += pack("l", self.diffuse)
		
		if self.fvf & D3DFVF_SPECULAR == D3DFVF_SPECULAR:
			ret += pack("l", self.specular)
		
		texCount = 0
		if self.fvf & D3DFVF_TEX1 == D3DFVF_TEX1: texCount = 1
		elif self.fvf & D3DFVF_TEX2 == D3DFVF_TEX2: texCount = 2
		elif self.fvf & D3DFVF_TEX3 == D3DFVF_TEX3: texCount = 3
		elif self.fvf & D3DFVF_TEX4 == D3DFVF_TEX4: texCount = 4
		elif self.fvf & D3DFVF_TEX5 == D3DFVF_TEX5: texCount = 5
		elif self.fvf & D3DFVF_TEX6 == D3DFVF_TEX6: texCount = 6
		elif self.fvf & D3DFVF_TEX7 == D3DFVF_TEX7: texCount = 7
		elif self.fvf & D3DFVF_TEX8 == D3DFVF_TEX8: texCount = 8

		for i in range(texCount):
			ret += pack("ff", 
				self.texcoord[i].x,
				self.texcoord[i].y)
		
		return ret


def FileSelectorCallback(filename):
	object = Object.GetSelected()[0]
	
	if object.type != 'Mesh':
		print object, 'is not mesh'
		return

	data = object.data

	# set fvf
	fvf = D3DFVF_XYZ
	if data.hasFaceUV(): fvf |= D3DFVF_TEX1

	# read fvf specified in property field
	for p in object.getAllProperties():
		name = p.name
		name.upper()
		try:
			if name.index('FVF') == 0:
				exec("fvf |=%s" % (p.data))
		except:
			pass
			
	# create new vertex class
	vertex = Direct3DVertex(fvf)
	
	# calculate world matrix
	matTransform = ScaleMatrix(-1, 4, Vector([0,1,0])) * RotationMatrix(-90, 4, "x")

	vertStride = len(vertex.getBytes())
	vertCount = len(data.faces) * 3
	indexCount = len(data.faces) * 3
	mtrlCount = len(data.getMaterials())
	if mtrlCount < 1: mtrlCount = 1
	
	print "FVF:", vertex.fvf
	print "vertex stride:", vertStride
	print "vertex count:", vertCount
	print "index count:", indexCount
	print "material count:", mtrlCount

	file = open(filename, "wb")
	file.write(pack('llllll', 
		0,
		vertex.fvf,
		vertStride,
		vertCount,
		indexCount,
		mtrlCount))
	
	# for each materials
	for mtrlIndex in range(mtrlCount):
	
		# vertex buffer
		for face in data.faces:
		
			if face.materialIndex != mtrlIndex:
				continue
				
			for i in range(3):
				
				# transform vertex coordinate
				pos = face.v[i].co * object.matrix * matTransform
				vertex.position = pos
				
				# texture coordinate
				if data.hasFaceUV():
					vertex.texcoord[0] = Vector(face.uv[i][0], 1 - face.uv[i][1])
				
				# normal
				if fvf & D3DFVF_NORMAL:
					vertex.normal = face.v[i].no * (object.matrix * matTransform).rotationPart()
				
				# weight
				if fvf & D3DFVF_XYZB1:
					pairs = data.getVertexInfluences(face.v[i].index)
					for pairIndex in range(len(pairs)):
						vertex.weight[pairIndex] = pairs[pairIndex][1]
				
				file.write(vertex.getBytes())
	
	# index buffer
	for i in range(indexCount):
		file.write(pack("h", indexCount - i - 1))

	# groups
	prev = 0
	for mtrlIndex in range(mtrlCount):
		cnt = 0
		
		# count how many vertices are assigned to this group
		for face in data.faces:
			if face.materialIndex == mtrlIndex:
				cnt += 3
		
		# MinIndex, NumVertices, StartIndex, NumPrimitives
		file.write(pack('llll', prev, cnt, prev, cnt / 3))
		
		prev += cnt
		
	
	file.close()
	


Window.FileSelector(FileSelectorCallback, "OK", Blender.sys.makename(ext='.xyz'))