import Blender
from Blender import *
from Blender.Mathutils import *
from struct import *


FVF = {}
FVF["D3DFVF_XYZ"]              = 0x002
FVF["D3DFVF_XYZRHW"]           = 0x004
FVF["D3DFVF_XYZB1"]            = 0x006
FVF["D3DFVF_XYZB2"]            = 0x008
FVF["D3DFVF_XYZB3"]            = 0x00a
FVF["D3DFVF_XYZB4"]            = 0x00c
FVF["D3DFVF_XYZB5"]            = 0x00e
FVF["D3DFVF_XYZW"]             = 0x4002
FVF["D3DFVF_NORMAL"]           = 0x010
FVF["D3DFVF_PSIZE"]            = 0x020
FVF["D3DFVF_DIFFUSE"]          = 0x040
FVF["D3DFVF_SPECULAR"]         = 0x080
FVF["D3DFVF_TEX0"]             = 0x000
FVF["D3DFVF_TEX1"]             = 0x100
FVF["D3DFVF_TEX2"]             = 0x200
FVF["D3DFVF_TEX3"]             = 0x300
FVF["D3DFVF_TEX4"]             = 0x400
FVF["D3DFVF_TEX5"]             = 0x500
FVF["D3DFVF_TEX6"]             = 0x600
FVF["D3DFVF_TEX7"]             = 0x700
FVF["D3DFVF_TEX8"]             = 0x800
FVF["D3DFVF_LASTBETA_UBYTE4"]  = 0x1000
FVF["D3DFVF_POSITION_MASK"]		= 0x400E
FVF["D3DFVF_TEXCOUNT_MASK"]		= 0xf00


class Direct3DVertex:
	def __init__(self, fvf):
		self.fvf = fvf
		self.position = Vector(0, 0, 0, 0)
		self.normal = Vector(0, 0, 0)
		self.diffuse = 0xFFFFFFFF
		self.specular = 0xFFFFFFFF
		self.weight = [0, 0, 0, 0, 0]
		self.texcoord = []
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.texcoord.append(Vector(0, 0))
		self.indices = 0
	def getBytes(self):
		ret = ""
		posBits = self.fvf & FVF["D3DFVF_POSITION_MASK"]
		texBits = self.fvf & FVF["D3DFVF_TEXCOUNT_MASK"]
		
		ret += pack("fff", 
			self.position.x, 
			self.position.y, 
			self.position.z)
		
		if posBits == FVF["D3DFVF_XYZW"]:
			ret += pack("f", self.position.w)
		
		wCount = 0
		if posBits == FVF["D3DFVF_XYZB5"]: wCount = 5
		elif posBits == FVF["D3DFVF_XYZB4"]: wCount = 4
		elif posBits == FVF["D3DFVF_XYZB3"]: wCount = 3
		elif posBits == FVF["D3DFVF_XYZB2"]: wCount = 2
		elif posBits == FVF["D3DFVF_XYZB1"]: wCount = 1

		if self.fvf & FVF["D3DFVF_LASTBETA_UBYTE4"] == FVF["D3DFVF_LASTBETA_UBYTE4"]:
			wCount -= 1
			
		for i in range(wCount):
			ret += pack("f", self.weight[i])
			
		if self.fvf & FVF["D3DFVF_LASTBETA_UBYTE4"] == FVF["D3DFVF_LASTBETA_UBYTE4"]:
			ret += pack("l", self.indices)
			
		if self.fvf & FVF["D3DFVF_NORMAL"] == FVF["D3DFVF_NORMAL"]:
			ret += pack("fff", 
				self.normal.x, 
				self.normal.y, 
				self.normal.z)
		
		if self.fvf & FVF["D3DFVF_DIFFUSE"] == FVF["D3DFVF_DIFFUSE"]:
			ret += pack("l", self.diffuse)
		
		if self.fvf & FVF["D3DFVF_SPECULAR"] == FVF["D3DFVF_SPECULAR"]:
			ret += pack("l", self.specular)
		
		texCount = 0
		if texBits == FVF["D3DFVF_TEX1"]: texCount = 1
		elif texBits == FVF["D3DFVF_TEX2"]: texCount = 2
		elif texBits == FVF["D3DFVF_TEX3"]: texCount = 3
		elif texBits == FVF["D3DFVF_TEX4"]: texCount = 4
		elif texBits == FVF["D3DFVF_TEX5"]: texCount = 5
		elif texBits == FVF["D3DFVF_TEX6"]: texCount = 6
		elif texBits == FVF["D3DFVF_TEX7"]: texCount = 7
		elif texBits == FVF["D3DFVF_TEX8"]: texCount = 8

		for i in range(texCount):
			ret += pack("ff", 
				self.texcoord[i].x,
				self.texcoord[i].y)
		
		return ret


# returns sorted bone list
def SortBoneList(armObj):
	armData = armObj.data
	boneList = []
	
	# enum top-level bones
	for bone in armData.bones.values():
		if not bone.parent:
			boneList.append(bone)
	
	boneIndex = 0
	for bone in boneList:
		for child in bone.children:
			boneList.insert(boneIndex + 1, child)
			boneIndex += 1
		boneIndex += 1
	
	return boneList


# returns bone index
def GetBoneIndex(boneList, boneName):
	for bone in boneList:
		if bone.name == boneName:
			return boneList.index(bone)
	return -1


def FileSelectorCallback(filename):
	object = Object.GetSelected()[0]
	
	if object.type != 'Mesh':
		print object, 'is not mesh'
		return

	fvf = 0
	meshVersion = 0
	meshObject = object
	meshData = meshObject.data
	armObject = 0
	armData = 0
	weightCount = 0
	enableBlendIndices = 0

	# if parent is armature, set output mesh version to 2
	if object.parentType == Object.ParentTypes.ARMATURE:
		meshVersion = 2
		armObject = object.parent
		armData = armObject.data		
		boneList = SortBoneList(armObject)
		weightCount = 4
		enableBlendIndices = 1
		fvf = FVF["D3DFVF_XYZB4"] | FVF["D3DFVF_LASTBETA_UBYTE4"]
	else:
		fvf = FVF["D3DFVF_XYZ"]

	# if mesh has UV coords, set TEX1 bit
	if meshData.hasFaceUV(): fvf |= FVF["D3DFVF_TEX1"]

	# read fvf specified in property field
	for p in meshObject.getAllProperties():
		name = p.name
		name.upper()
		try:
			if name.index('D3DFVF') == 0:
				fvf |= FVF[name]
		except:
			pass
			
	# create new vertex class
	vertex = Direct3DVertex(fvf)
	
	# 
	matTransform = RotationMatrix(-90, 4, 'x')
	
	# calculate world matrix
	vertTransform = ScaleMatrix(-1, 4, Vector([0,1,0])) * matTransform

	invZ = Matrix([1,0,0,0], [0,1,0,0], [0,0,-1,0], [0,0,0,1])
	
	vertStride = len(vertex.getBytes())
	vertCount = len(meshData.faces) * 3
	indexCount = len(meshData.faces) * 3
	mtrlCount = len(meshData.getMaterials())
	if mtrlCount < 1: mtrlCount = 1
	
	print "---------------------------------"
	print "version: ", meshVersion
	print "FVF:", vertex.fvf
	print "vertex stride:", vertStride
	print "vertex count:", vertCount
	print "index count:", indexCount
	print "material count:", mtrlCount

	file = open(filename, "wb")
	file.write(pack('llllll', 
		meshVersion,
		vertex.fvf,
		vertStride,
		vertCount,
		indexCount,
		mtrlCount))

	# extra information (version 2+)
	if meshVersion >= 2:
		file.write(pack("l", len(boneList)))
	
	# for each materials
	for mtrlIndex in range(mtrlCount):
	
		# vertex buffer
		for face in meshData.faces:
		
			if face.materialIndex != mtrlIndex:
				continue
				
			for i in range(3):
				
				# transform vertex coordinate
				pos = face.v[i].co * meshObject.matrix * vertTransform
				vertex.position = pos
				
				# texture coordinate
				if meshData.hasFaceUV():
					vertex.texcoord[0] = Vector(face.uv[i][0], 1 - face.uv[i][1])
				
				# normal
				if fvf & FVF["D3DFVF_NORMAL"]:
					vertex.normal = face.v[i].no * (meshObject.matrix * vertTransform).rotationPart()
				
				# weight
				if fvf & FVF["D3DFVF_XYZB1"]:
					pairs = meshData.getVertexInfluences(face.v[i].index)
					for pairIndex in range(len(pairs)):
						vertex.weight[pairIndex] = pairs[pairIndex][1]
						
				# blending weights
				if enableBlendIndices:
					pairs = meshData.getVertexInfluences(face.v[i].index)
					
					# sort pairs using weight(2nd element) as key
					pairs.sort(cmp=lambda x, y: cmp(x[1], y[1]), reverse=True)
					
					# pick top weights
					vertex.indices = 0
					sum = 0
					j = 0
					while j < weightCount and j < len(pairs):
						vertex.indices |= (GetBoneIndex(boneList, pairs[j][0]) << (j * 8))
						vertex.weight[j] = pairs[j][1]
						sum += pairs[j][1]
						j += 1
						
					# normalize weights
					j = 0
					while j < weightCount and j < len(pairs):
						vertex.weight[j] /= sum
						j += 1
					
				
				file.write(vertex.getBytes())
	
	# index buffer
	for i in range(0, indexCount, 3):
		file.write(pack("hhh", i + 2, i + 1, i))

	# groups
	prev = 0
	for mtrlIndex in range(mtrlCount):
		cnt = 0
		
		# count how many vertices are assigned to this group
		for face in meshData.faces:
			if face.materialIndex == mtrlIndex:
				cnt += 3
		
		# MinIndex, NumVertices, StartIndex, NumPrimitives
		file.write(pack('llll', prev, cnt, prev, cnt / 3))
		
		print "Material%d: %d vertices" % (mtrlIndex, cnt)
		
		prev += cnt
		
	# bone list
	if armData:
		
		for bone in boneList:
						
			# if root bone
			if bone.parent:
				parentIndex = boneList.index(bone.parent)
				parentMatrix = bone.parent.matrix['ARMATURESPACE'].copy()
				boneMatrix = bone.matrix['ARMATURESPACE'] * parentMatrix.invert()
			else:
				parentIndex = -1
				boneMatrix = bone.matrix['ARMATURESPACE'] * armObject.matrix * matTransform
				
			# convert coordinate system of transform matrix
			boneMatrix = invZ * boneMatrix * invZ
			
			# bone data
			#
			#   char[32]   bone name
			#   long        parent's index or -1
			#   float[16]   bone matrix
			#
			file.write(pack('32sl', bone.name, parentIndex))

			# matrix
			for row in boneMatrix:
				for col in row:
					file.write(pack('f', col))
	
	file.close()


Window.FileSelector(FileSelectorCallback, "OK", Blender.sys.makename(ext='.xyz'))
