import Blender
import math
from Blender import *
from Blender.Mathutils import *

def main():

	if len(Object.GetSelected()) <= 0:
		return
	
	curveObj = Object.GetSelected()[0]
	if curveObj.type != 'Curve':
		return
	
	curveData = curveObj.data
	
	bevObj = curveData.bevob
	if not bevObj:
		return
	
	bevData = bevObj.data
	
	dummy = Object.New('Empty')
	if not dummy:
		print "Failed to create empty object!"
		return
	Scene.GetCurrent().objects.link(dummy)
	
	meshData = Mesh.New()

	for curve in curveData:
		dummy.setLocation(0, 0, 0)

	# add FOLLOWPATH constraint to Empty
	const = dummy.constraints.append(Constraint.Type.FOLLOWPATH)
	const[Constraint.Settings.TARGET] = curveObj
	const[Constraint.Settings.FORWARD] = Constraint.Settings.TRACKY

	pointCount = curveData.getPathLen() #len(curve) * curveData.resolu
	pathLen = curveData.getPathLen()
	deltaFrame = pathLen / pointCount
	if deltaFrame < 1: deltaFrame = 1

	for i in range(0, pointCount + 1):
		Blender.Redraw()
		
		if 0 < i:
			Blender.Set('curframe', 
				(i - 1) * deltaFrame)
		else:
			Blender.Set('curframe', 
				i * deltaFrame)
		p0 = Vector(dummy.getLocation('worldspace'))

		Blender.Set('curframe', 
			i * deltaFrame)
		p1 = Vector(dummy.getLocation('worldspace'))

		Blender.Set('curframe', 
			(i + 1) * deltaFrame)
		p2 = Vector(dummy.getLocation('worldspace'))
		
		# calculate interpolated direction vector		
		n0 = p1 - p0
		n1 = p2 - p1
		N = (n0 + n1) * 0.5
		N.normalize()

		# calculate axis vectors
		X = Vector(1, 0, 0)
		Y = Mathutils.CrossVecs(X, N)
		X = Mathutils.CrossVecs(N, Y)
		X.normalize()
		Y.normalize()
		
		# add verts
		for seg in range(bevData.resolu):
			x = math.cos(math.pi*2 * seg/bevData.resolu)
			y = math.sin(math.pi*2 * seg/bevData.resolu)
			meshData.verts.extend(p1 + x * X + y * Y)
		
		# add faces
		if i > 0:
			faces = [[0, 1, 2, 3]]
			
			for seg in range(bevData.resolu):
				start = i * bevData.resolu
				faces[0][0] = start + seg
				faces[0][1] = start + (seg + 1) % (bevData.resolu)
				faces[0][2] = faces[0][1] - bevData.resolu
				faces[0][3] = faces[0][0] - bevData.resolu
				meshData.faces.extend(faces)

	
	# add mesh to scene
	meshObj = Scene.GetCurrent().objects.new(meshData)
	meshObj.wireMode = 1
	

	# make all faces triangle				
	meshData.quadToTriangle()

	
	# delete dummy empty object
	Scene.GetCurrent().objects.unlink(dummy)
	
	
	# set vertex weights
	meshData.addVertGroup('0')
	for i in range(pointCount + 1):
		verts = []
		
		for j in range(bevData.resolu):
			verts.append(i * bevData.resolu + j)
		
		meshData.assignVertsToGroup(
			'0',
			verts,
			float(i) / pointCount,
			1)
	
	
	# add vertex format to properties
	meshObj.addProperty('FVF', 'D3DFVF_XYZB1')
	meshObj.addProperty('FVF', 'D3DFVF_NORMAL')
	

	# switch selecting object
	curveObj.select(0)
	meshObj.select(1)

	
	Scene.GetCurrent().update()
	Blender.Redraw()
	

main()