
shared float3		vViewPosition;
shared float4x4		mViewProj;
shared float4x4		mWorldViewProj;
shared float		fFogStart;
shared float		fFogEnd;
shared bool			bFogEnable;

float4x4			mWorld;

sampler				sSampler0 : register(s0);


//==============================================================================
// ^C`悷钸_VF[_[
//==============================================================================
void VS_Tile(
	float4		inPosition	: POSITION,
	float2		inTex0		: TEXCOORD0,
	out float4	outPosition	: POSITION,
	out float2	outTex		: TEXCOORD0,
	out float	outFog		: FOG
)
{
	float4 pos = mul(inPosition, mWorld);
	outPosition = mul(pos, mViewProj);
	outTex = inTex0;
	
	if (bFogEnable)
	{
		outFog = (fFogEnd - length((float3)pos - vViewPosition)) / (fFogEnd - fFogStart);
	}
	else
	{
		outFog = 1;
	}
}


//==============================================================================
// ^C`悷钸_VF[_[
//==============================================================================
void VS_ProjectedTile(
	float4		inPosition	: POSITION,
	float2		inTex0		: TEXCOORD0,
	out float4	outPosition	: POSITION,
	out float2	outTex		: TEXCOORD0
)
{
	outPosition = mul(inPosition, mWorldViewProj);
	outTex = inTex0;
}


//==============================================================================
// 2DXvCg`悷钸_VF[_[
//==============================================================================
void VS_NormalSprite(
	float4		inPosition	: POSITION,
	float4		inColor		: COLOR0,
	float2		inTex0		: TEXCOORD0,
	out float4	outPosition	: POSITION,
	out float4	outColor	: COLOR0,
	out float2	outTex		: TEXCOORD0
)
{
	outPosition = mul(inPosition, mWorldViewProj);
	outColor = inColor;
	outTex = inTex0;
}


//==============================================================================
// GtFNg̃sNZVF[_[
//==============================================================================
float4 PS_GlowEffect(float4 inColor : COLOR0, float2 inTex : TEXCOORD0) : COLOR
{
	float4 texColor = tex2D(sSampler0, inTex);
	return texColor + inColor;
}


// ^Cp̃eNjbN
technique Tile
{
	pass
	{
		FogEnable = bFogEnable;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_2_0 VS_Tile();
		PixelShader = NULL;
	}
}


// ^Cp̃eNjbN
technique ProjectedTile
{
	pass
	{
		FogEnable = bFogEnable;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_2_0 VS_ProjectedTile();
		PixelShader = NULL;
	}
}


// ʏ
technique Normal
{
	pass
	{
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATEREQUAL;
		AlphaRef = 1;
		VertexShader = compile vs_2_0 VS_NormalSprite();
		PixelShader = NULL;
	}
}


// Z
technique Add
{
	pass
	{
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATEREQUAL;
		AlphaRef = 1;
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_2_0 VS_NormalSprite();
		PixelShader = NULL;
	}
}


// Z
technique Multiply
{
	pass
	{
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATEREQUAL;
		AlphaRef = 1;
		AlphaBlendEnable = TRUE;
		SrcBlend = DESTCOLOR;
		DestBlend = ZERO;
		VertexShader = compile vs_2_0 VS_NormalSprite();
		PixelShader = NULL;
	}
}


// At@uh
technique Alpha
{
	pass
	{
		AlphaTestEnable = FALSE;
		AlphaBlendEnable = TRUE;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_2_0 VS_NormalSprite();
		PixelShader = NULL;
	}
}


// 
technique Glow
{
	pass
	{
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATEREQUAL;
		AlphaRef = 1;
		VertexShader = compile vs_2_0 VS_NormalSprite();
		PixelShader = compile ps_2_0 PS_GlowEffect();
	}
}


